﻿(function ($) {
    // Get the Checkbox's or Radio's parent Label control
    var _getLabel = function ($input) {
        var $label = $input.parent();
        if ($label.is('label')) {
            return $label.css({ cursor: 'pointer' });
        } else {
            return $('<label>', { 'class': $input.is('input:checkbox') ? 'checkbox' : 'radio', css: { cursor: 'pointer' } })
                        .insertBefore($input)
                        .append($input);
        }
    };

    // Set the Checkbox's or Radio's status
    var _setInputStatus = function ($input) {
        var $icon = $input.siblings('i');
        $input.attr('checked') ? $icon.addClass('checked') : $icon.removeClass('checked');
    }

    // Set the Select's status
    var _setSelectStatus = function ($select) {
        var $wrapper = $select.prev('div.dropdown-list');
        var $ul = $('ul', $wrapper);
        var $aLink = $('a[data-value="' + $('option:selected', $select).val() + '"]', $ul);

        $('a.active', $ul).removeClass('active');
        if ($aLink && $aLink.length) {
            $aLink.addClass('active');
            $('button.dropdown-toggle span:eq(0)', $wrapper).html($aLink.html());
            $ul.scrollTop($aLink[0].offsetTop - 1);
        } else {
            $('button.dropdown-toggle span:eq(0)', $wrapper).html('&nbsp;');
            $ul.scrollTop($ul[0].offsetTop - 1);
        }
    }

    // Set the Select's value
    var _setSelectValue = function ($wrapper, $aLink) {
        if ($aLink) {
            var $select = $wrapper.siblings('select');
            var $ul = $aLink.parents('ul');

            $('option:selected', $select).attr({ selected: false });
            $('option[value="' + $aLink.attr('data-value') + '"]', $select).attr({ selected: true });

            $('a.active', $ul).removeClass('active');
            $aLink.addClass('active');
            $('button.dropdown-toggle span:eq(0)', $wrapper).html($aLink.html());
        }
    };

    // Reset the Checkbox / Radio / Select
    var _reset = function () {
        $('form input:checkbox, form input:radio, form select').trigger('transformReset');
    };

    // Transform Checkbox / Radio
    $.fn.transformInput = function () {
        return this.each(function () {
            var $input = $(this);
            if ($input.data('transformed')) { return; }

            var _defaultChecked = false;
            var $label = _getLabel($input);
            var $icon = $('<i>', { 'class': $input.is('input:checkbox') ? 'icon-sfa-checkbox' : 'icon-sfa-radio' })
                .insertBefore($input);
            $input.attr('checked') && $icon.addClass('checked') && (_defaultChecked = true);
            $input.attr('disabled') && $icon.addClass('disabled');

            $label
                .hover(function () { $icon.addClass('hover'); return $label; }, function () { $icon.removeClass('hover').removeClass('active'); return $label; })
                .mousedown(function (event) { if (event.which == 1) { $icon.addClass('active'); } return $label; })
                .mouseup(function (event) { $icon.removeClass('active'); return $label; });
            $input
                .data({ transformed: true })
                .css({ opacity: 0, position: 'absolute', left: -99999 })
                .change(function () {
                    if (!$input.attr('disabled')) {
                        if ($input.is('input:radio')) {
                            var _name = $input.attr('name');
                            if (_name != undefined && _name != '') {
                                $('input[name="' + _name + '"]', this.form).not($input).attr({ checked: false }).transformResetStatus();
                            }
                            $input.attr({ checked: true });
                        }
                        $input.transformResetStatus();
                    }

                    return $input;
                })
                .on({
                    transformReset: function () {
                        $input.attr({ checked: _defaultChecked }).transformResetStatus();
                    }
                });
        });
    };

    // Transform Select
    $.fn.transformSelect = function () {
        return this.each(function () {
            var $select = $(this);
            if ($select.data('transformed')) {
                $select.removeClass('hide').prev("div.dropdown-list").remove();
            }

            var _$defaultSelected = $('option:selected', $select);
            if ($select.attr('multiple')) {
                /*
                var _filter = $select.data('filter');
                if (_filter === undefined || _filter === "") {
                    _filter = $select.attr('filter');
                    if (_filter === undefined || _filter === "") { _filter = null; }
                }
                */
            } else {
                var $button = $('<button>', { type: 'button', 'class': 'btn dropdown-toggle', 'data-toggle': 'dropdown' })
                    .append($('<span>', { 'class': 'text-left pull-left' }))
                    .append($('<i>', { 'class': 'caret pull-right' }))
                    .click(function () {
                        $button.dropdown();
                        return $button;
                    });
                $select.attr('disabled') && $button.addClass('disabled').attr({ disabled: true });

                var $selectALink;
                var _addOption = function ($option, inGroup) {
                    var $aLink = $('<a>', { href: 'javascript:void(0)', 'data-value': $option.val(), html: $option.html() === '' ? '&nbsp;' : $option.html() })
                        .click(function () {
                            if (!$aLink.data('disabled')) {
                                _setSelectValue($wrapper, $aLink);
                                $ul.scrollTop($aLink[0].offsetTop - 1);

                                $select.trigger('change');

                                if ($.validator)
                                    $select.valid();
                            }

                            return $aLink;
                        });
                    inGroup && $aLink.addClass('groupopt');
                    $option.attr('disabled') && $aLink.addClass('disabled').data({ disabled: true });
                    $option.attr('selected') && ($selectALink = $aLink);

                    $ul.append($('<li>').append($aLink));
                }

                var $ul = $('<ul>', { 'class': 'dropdown-menu' });
                $select
                    .children()
                    .each(function () {
                        var $obj = $(this);
                        if ($obj.is('optgroup')) {
                            $ul.append(
                                $('<li>').append(
                                    $('<label>', { 'class': 'optgroup', html: $obj.attr('label') })
                                )
                            );
                            $('option', $obj).each(function () {
                                _addOption($(this), true);
                            });
                        } else if ($obj.is('option')) {
                            _addOption($obj);
                        }
                    });

                var selectClass = $select.attr('class');
                var $wrapper = $('<div>', { 'class': 'dropdown-list btn-group ' + selectClass })
                    .append($button)
                    .append($ul)
                    .insertBefore($select);
                if (!selectClass || !/span/.test(selectClass)) {
                    $wrapper.css({ width: '220px' });
                }
                if ($select[0].style.display === 'none')
                    $wrapper.hide();

                _setSelectValue($wrapper, $selectALink);    // Necessary!

                $select
                    .addClass('hide')
                    .data({ transformed: true })
                    .on({
                        hide: function () {
                            $wrapper.hide();
                            _setSelectStatus($select);
                        },
                        show: function () {
                            $wrapper.show();
                            _setSelectStatus($select);
                        },
                        transformReset: function () {
                            $('option', $select).not(_$defaultSelected.attr({ selected: true })).attr({ selected: false });
                            $select.transformResetStatus();
                        }
                    });
            }
        });
    };

    // Reset Checkbox's & Radio's & Select's Status
    $.fn.transformResetStatus = function () {
        return this.each(function () {
            var $obj = $(this);
            if ($obj.is('input')) {
                $obj.data('transformed') && _setInputStatus($obj);
            } else if ($obj.is('select')) {
                $obj.data('transformed') && _setSelectStatus($obj);
            }
        });
    };

    // Override Transformed Selct's hide & show event
    var _oldhide = $.fn.hide;
    var _oldshow = $.fn.show;
    $.fn.hide = function (speed, callback) {
        if (this.is('select') && this.data('transformed')) {
            this.trigger('hide');
        } else {
            _oldhide.apply(this, arguments);
        }
        return this;
    };
    $.fn.show = function (speed, callback) {
        if (this.is('select') && this.data('transformed')) {
            this.trigger('show');
        } else {
            _oldshow.apply(this, arguments);
        }
        return this;
    };

    var transform = function ($body) {
        $('input:checkbox, input:radio', $body).transformInput();
        $('select', $body).transformSelect();

        $('form').bind({
            reset: function () {
                window.setTimeout(function () {
                    _reset();
                }, 10);
            }
        });
    };

    $.fn.transform = function () {
        transform(this);
    }
})(jQuery);